/* .codeXmlLib.js
 * $Revision: 7022 $
 * 
 * helper functions/objects for JavaScript translation of CodeXML rules.imf
 *
 * Note: functions are provided for some transforms; others use inline javascript:
 *     extract:          function
 *     extractRange:     function
 *     translateFirst:   t.buffer = t.buffer.replace(/exp/, "new");
 *     translateAll:     t.buffer = t.buffer.replace(/exp/g, "new");
 *     deleteFirst:      t.buffer = t.buffer.replace(/exp/, "");
 *     deleteAll:        t.buffer = t.buffer.replace(/exp/g, "");
 *     deleteRange:      t.buffer = t.buffer.substr(0, pos) + t.buffer.substr(pos + len);
 *     tag:              codeXml_tag = buf;
 *     accept:           reader.output("\x01X\x1e" + codeXml_tag + "/" + buf + "\x04");
 *     passthrough:      reader.output(buf);
 *     invalidateDecode: codeXml_valid = false; return;
 *     insert:           t.buffer = t.buffer.substr(0, pos) + buf + t.buffer.substr(pos);
 *     (special pos:
 *         extracted:    t.found.index
 *         end:          t.buffer.length
 *     )
 */



var codeXml_topRule    = 1; /* constant */
var codeXml_rules      = new Array();
var codeXml_valid      = true;
var codeXml_tag        = "";
var codeXml_activeRule = codeXml_topRule;



function codeXml_activateRule(ruleNumber, activate)
{
    for(i = 0; i < codeXml_rules.length; ++i)
        if( codeXml_rules[i] )
            codeXml_rules[i].active = activate;
}



function codeXml_extract(t, regex)
{
    t.found = t.buffer.match(regex);
    if( !t.found )
        t.extracted = "";
    else
    {
        t.extracted = t.found[0];
        t.buffer    = t.buffer.substr(0, t.found.index)
             + t.buffer.substr(t.found.index + t.found[0].length)
             ;
    }

    return t;
}



function codeXml_extractRange(t, pos, len)
{
    t.extracted = t.buffer.substr(pos, len);

    t.buffer = t.buffer.substr(0, pos)
             + t.buffer.substr(pos + len)
             ;

    return t;
}



reader.filterDecode = function(decode)
{
    var buffer = decode.data;

    codeXml_valid      = true;
    codeXml_tag        = "";
    codeXml_activeRule = codeXml_topRule;

    /* rule selection/execution
     * TODO: handle child rules
     */
    while( buffer.length > 0 )
    {
        var i;
        var first = { rule:codeXml_topRule, pos:buffer.length };
        var pos;
        for(i = 0; i < codeXml_rules.length; ++i)
        {
            if(    codeXml_rules[i]
               &&  codeXml_rules[i].active
               &&  codeXml_rules[i].parent == codeXml_activeRule
              )
            {
                pos = buffer.search(codeXml_rules[i].pattern);
                if( pos >= 0  &&  pos < first.pos )
                {
                    first.rule = i;
                    first.pos  = pos;
                }
            }
        }

        if( first.rule <= codeXml_topRule )
            break;

        var found = buffer.match(codeXml_rules[first.rule].pattern;
        buffer = buffer.substr(0, found.index)
               + buffer.substr(found.index + found[0].length, buffer.length)
               ;
        codeXml_rules[first.rule].func(found[0]);

        if( !codeXml_valid )
            return false;
    }

    /* implicit passthrough rule
     */
    if( buffer.length > 0 )
        reader.output(buffer);

    return true;
}



/*EOF*/
