// configure.js
// CR3x configuration utility
// Copyright (c) 2013 The Code Corporation. All rights reserved.



include(".settings.js");



// registers
var commProtocol    = 0x08;

var comm_version0   =  0;
var comm_raw        =  1;
var comm_version1   =  2;

var commMode        = 0x1B;

var comm_kb         =  0;    // not supported in cr8000
var comm_uart       =  1;
var comm_usbKb      =  2;
var comm_usbIbm     =  3;
var comm_bt         =  4;
var comm_usbNative  =  5;
var comm_usbVcom    =  6;
var comm_usbHidPos  =  7;
var comm_dynamic    =  8;
var comm_wifi       =  9;
var comm_btKb       = 10;
var comm_iap2ExtAcc = 11;
var comm_iap2Kb     = 12;
var comm_usbPassthrough = 13;

var commExpectResponse = 0x42;

var rfMinLinkQuality_fp24_8 = 0xb2;

var button1 = 0x39;     // right button
var button2 = 0x3a;     // left button
var button4 = 0x3c;     // handle button

var sxgaBoth     = 0x1c;
var sxgaNear     = 0x1d;
var sxgaFar      = 0x1e;
var vgaBoth      = 0x1f;
var vgaNear      = 0x20;
var vgaFar       = 0x21;
var adaptiveBoth = 0x22;
var adaptiveNear = 0x23;
var adaptiveFar  = 0x24;

// menus
main                 = new gui.Menu();
main.app             = new gui.Menu();
main.app.options     = new gui.Menu();
main.comm            = new gui.Menu();
main.comm.interfaces = new gui.Menu();
main.info            = new gui.Menu();
main.optimization    = new gui.Menu();
main.system          = new gui.Menu();



/*****************************************************************************/
// Application Submenu for Main Form
/*****************************************************************************/
main.app.currentFile = new String();

main.app.options.version_onClick = function()
{
    var re = new RegExp("\\$" + "Revision: .*\\$"); // separate $ and Version to avoid RCS keyword substitution
    var file = storage.read(main.app.currentFile);

    var ret = file.match(re);
    if( ret == null )
    {
        ret = "Unknown version";
    }
    else
    {
        ret = ret.toString();
        ret = ret.substr(1, ret.length - 1 - 2/* $*/);
    }

    dialog = new gui.Text(ret);
    dialog.caption = main.app.currentFile;
    dialog.onCancel = function() { gui.showSubMenu(main.app.options, main.app); }
    dialog.onOk = dialog.onCancel;
    dialog.leftSoftkey = new gui.Softkey("OK", dialog.onCancel);

    gui.showDialog(dialog);
}



main.app.deleteMenu = function()
{
    gui.confirm(main.app.confirmDelete, null, "Are you sure?");
}



main.app.confirmDelete = function()
{
    storage.erase(main.app.currentFile);
    reader.runScript(".configure.js");
}



main.app.options.append( new gui.MenuItem("Run",            function() { if( !reader.runScript(main.app.currentFile) ) alert(null, "Run script failed!"); }) );
main.app.options.append( new gui.MenuItem("Version",        main.app.options.version_onClick) );

if( !reader.readSetting(setting_disableDeleteOption) )
    main.app.options.append( new gui.MenuItem("Delete",     main.app.deleteMenu) );

main.app.options.append( new gui.MenuItem("Set as default", function() { storage.write(".default.js", "include(\"" + main.app.currentFile + "\");"); reader.runScript(".default.js");}) );



main.app_onClick = function()
{
    main.app.options.caption = this.text;
    main.app.currentFile = main.app.appArray[main.app.titleArray.indexOf(this.text)];
    gui.showSubMenu(main.app.options, main.app)
}



main.app.caption    = "Applications";
main.app.appArray   = new Array();
main.app.titleArray = new Array();

var app = storage.findFirst(/^[^.].+\.js$/);

if( app != null )
{
    main.app.appArray.push(app);

    while( (app = storage.findNext()) != null )
    {
        main.app.appArray.push(app);
    }

    //Parse for title keyword and add to appArray for sorting
    for( var i = 0; i < main.app.appArray.length; ++i )
    {
        var file = storage.read(main.app.appArray[i]);

        var ret = file.match(/\$Title: .*\$/);
        if( ret == null )
        {
            ret = main.app.appArray[i];
        }
        else
        {
            ret = ret.toString();
            ret = ret.replace(/\$Title: /, "");
            ret = ret.replace(/\$$/,"");
        }

        main.app.appArray[i] = ret + "\x1e" + main.app.appArray[i];
    }

    //sort by title
    main.app.appArray.sort();

    //Split appArray into title array and appArray
    for( var j = 0; j < main.app.appArray.length; j++ )
    {
        main.app.titleArray[j] = main.app.appArray[j].substring(0, main.app.appArray[j].indexOf("\x1e"));
        main.app.appArray[j] = main.app.appArray[j].replace(/^.+\x1e/, "");
    }

    //Display titles on form
    for( var k = 0; k < main.app.titleArray.length; k++ )
        main.app.append( new gui.MenuItem(main.app.titleArray[k], main.app_onClick) );
}
/*****************************************************************************/



/*****************************************************************************/
// Communications Submenu
/*****************************************************************************/
main.comm.interfaces.promptSaveSetting = function(postPromptFunc)
{
	gui.confirm( postPromptFunc, null, "Settings changed.\n\nSave now?");
}



main.comm.interfaces.bt_onClick = function()
{
    main.comm.interfaces.promptSaveSetting(main.comm.interfaces.change_tobtMode);
}



main.comm.interfaces.change_tobtMode = function()
{
	reader.processCommand("P", String.fromCharCode(commProtocol) + comm_version1);
    reader.processCommand("P", String.fromCharCode(commMode) + comm_bt);
    reader.processCommand("P", String.fromCharCode(commExpectResponse) + "1");
	reader.saveSettings();
    reader.runScript(".default.js");
}



include(".configure_main_comm_interface_usb.js");



main.comm.interfaces.usb_onClick = function()
{
    main.comm.interfaces.usb.initItemText();

    main.comm.interfaces.usb.caption = "USB";
    gui.showSubMenu(main.comm.interfaces.usb, main.comm.interfaces);
}



include(".configure_main_comm_interface_wifi.js");
main.comm.interfaces.wifi_onClick = function()
{
    main.comm.interfaces.wifi.caption = "WIFI";
	gui.showSubMenu(main.comm.interfaces.wifi, main.comm.interfaces);
}



main.comm.interfaces.initItemText = function()
{
	//USB comm mode is available by default.
    if(!main.comm.interfaces.usbItem)
    {
        main.comm.interfaces.usbItem = new gui.MenuItem(" USB", main.comm.interfaces.usb_onClick);
        main.comm.interfaces.append(main.comm.interfaces.usbItem);
    }

	//Add menu items based on the connected daughter board.
	if (reader.softwareVersion.substr(8, 4) == "blue")
	{
        if(!main.comm.interfaces.btItem)
        {
            main.comm.interfaces.btItem = new gui.MenuItem(" BT", main.comm.interfaces.bt_onClick);
            main.comm.interfaces.append(main.comm.interfaces.btItem);
        }
	}
	else if(reader.softwareVersion.substr(8, 4) == "wifi")
	{
        if(!main.comm.interfaces.wifiItem)
        {
            main.comm.interfaces.wifiItem  = new gui.MenuItem(" WIFI", main.comm.interfaces.wifi_onClick);
            main.comm.interfaces.append(main.comm.interfaces.wifiItem);
        }
	}
    switch( reader.readSetting(commMode) )
    {
		case comm_usbKb:
		case comm_usbIbm:
		case comm_usbNative:
		case comm_usbVcom:
			main.comm.interfaces.usbItem.text = "*" + main.comm.interfaces.usbItem.text.substr(1,3);
			break;

		case comm_bt:
		case comm_btkb:
			main.comm.interfaces.btItem.text = "*" + main.comm.interfaces.btItem.text.substr(1, 2);
			break;

		case comm_wifi:
			main.comm.interfaces.wifiItem.text = "*" + main.comm.interfaces.wifiItem.text.substr(1, 4);
			break;
    }
}



main.comm.interfaces_onClick = function()
{
    main.comm.interfaces.initItemText();

    main.comm.interfaces.caption = "Communication";
    gui.showSubMenu(main.comm.interfaces, main);
}



/*****************************************************************************/
// Info Submenu
/*****************************************************************************/
main.info.showDialog = function(dialog, caption)
{
    dialog.caption = caption;
    dialog.onCancel = function() { gui.showSubMenu(main.info, main); };
    dialog.onOk = dialog.onCancel;
    dialog.leftSoftkey = new gui.Softkey("OK", dialog.onCancel);

    gui.showDialog(dialog);
}



main.info.readerId_onClick = function()
{
    dialog = new gui.Text(reader.readerId);
    main.info.showDialog(dialog, this.text)
}



main.info.version_onClick = function()
{
    dialog = new gui.Text("App:     " +  reader.softwareVersion.substr(0, 4) + "\n"
                        + "Boot:    " + reader.softwareVersion.substr(4, 4) + "\n"
                        + "Radio:   " + reader.softwareVersion.substr(8, 4) + "\n"
                        + "OEM ID: " + reader.oemId);
    main.info.showDialog(dialog, this.text)
}



main.info.time_onClick = function()
{
    dialog = new gui.Text(Date().substr(0, 24));    // no time zone
    main.info.showDialog(dialog, this.text)
}



main.info.bdAddr_onClick= function()
{
    dialog = new gui.Text(reader.bdAddr != "none"? reader.bdAddr: "No radio");
    main.info.showDialog(dialog, this.text)
}



//Caption the menu and append items to the list.
main.info.caption = "Reader Info";
main.info.append( new gui.MenuItem("Firmware Ver:", main.info.version_onClick) );
main.info.append( new gui.MenuItem("Reader Serial #", main.info.readerId_onClick) );
main.info.append( new gui.MenuItem("Date/Time", main.info.time_onClick) );
main.info.append( new gui.MenuItem("Radio BD_ADDR", main.info.bdAddr_onClick) );
/*****************************************************************************/



/*****************************************************************************/
// System Submenu
main.system.backlightTime_onClick = function()
{
    form = new gui.Form();

    form.caption = "Backlight ms"

    form.time = new gui.Edit(reader.readSetting(0x0100).toString());

    form.append( new gui.Label("Set new value") );
    form.append(form.time);

    form.onOk = function()
    {
        reader.writeSetting(0x0100, parseInt(form.time.text, 10));
        gui.showSubMenu(main.system, main);
    }

    form.onCancel = function()
    {
        gui.showSubMenu(main.system, main);
    }

    gui.showForm(form);
}



main.system.setDateTime_onClick = function()
{
    form = new gui.Form();

    form.caption = "Set date/time "

    d = new Date();

    var month = new String();
    var day = new String();
    var hour = new String();
    var minute = new String();
    var second = new String();

    month = (d.getMonth() + 1).toString();
    if( month.length < 2 )
        month = "0" + month;

    day = d.getDate().toString();
    if( day.length < 2 )
        day = "0" + day;

    hour = d.getHours().toString();
    if( hour.length < 2 )
        hour = "0" + hour;

    minute = d.getMinutes().toString();
    if( minute.length < 2 )
        minute = "0" + minute;

    second = d.getSeconds().toString();
    if( second.length < 2 )
        second = "0" + second;

    form.year = new gui.Edit(d.getFullYear().toString());
    form.month = new gui.Edit(month);
    form.day = new gui.Edit(day);
    form.hour = new gui.Edit(hour);
    form.minute = new gui.Edit(minute);
    form.second = new gui.Edit(second);

    form.append( new gui.Label("Year(4 digit)") );
    form.append(form.year);
    form.append( new gui.Label("Month(2 digit)") );
    form.append(form.month);
    form.append( new gui.Label("Day(2 digit)") );
    form.append(form.day);
    form.append( new gui.Label("Hour(24)(2 digit)") );
    form.append(form.hour);
    form.append( new gui.Label("Minute(2 digit)") );
    form.append(form.minute);
    form.append( new gui.Label("Second(2 digit)") );
    form.append(form.second);

    form.onOk = function()
    {
        var re4 = new RegExp("^\\d{4}$");
        var re2 = new RegExp("^\\d{2}$");

        if( !re4.test(form.year.text)
         || !re2.test(form.month.text)
         || !re2.test(form.day.text)
         || !re2.test(form.hour.text)
         || !re2.test(form.minute.text)
         || !re2.test(form.second.text)
         )
        {
            gui.alert(main.system.incorrectDateFunc, "Incorrect format!");
        }
        else
        {
            reader.processCommand("@", form.year.text + form.month.text + form.day.text + form.hour.text + form.minute.text + form.second.text);

            gui.showSubMenu(main.system, main);
        }
    }

    form.onCancel = function()
    {
        gui.showSubMenu(main.system, main);
    }

    gui.showForm(form);
}



main.system.incorrectDateFunc = function()
{
	gui.showSubMenu(main.system, main);
}



main.system.confirmSave = function()
{
	reader.saveSettings();
	gui.showSubMenu(main.system, main);
}



main.system.saveSettings_onClick = function()
{
    gui.confirm(main.system.confirmSave, null, "Are you sure you want to save settings?");
}



main.system.confirmToDefault = function()
{
	reader.saveSettings();
	reader.runScript(".default.js");
}


main.system.restoreToDefault = function()
{
	reader.defaultSettings();
	gui.confirm(main.system.confirmToDefault, null, "Settings reset.\n\nSave now?");
}



main.system.factoryDefault_onClick = function()
{
    gui.confirm(main.system.restoreToDefault, null, "Are you sure you want to reset to factory defaults?");
}



main.system.resetToDefaults = function(save, initFunc)
{
    if( save )
    {
        reader.saveSettings();
        reader.runScript(".default.js");
    }
    else
    {
        initFunc();
    }
}



main.system.initSystem = function()
{
    gui.showSubMenu(main.system, main);
}



main.system.caption = "System";
main.system.append( new gui.MenuItem("Backlight time", main.system.backlightTime_onClick) );
main.system.append( new gui.MenuItem("Set date/time", main.system.setDateTime_onClick) );
main.system.append( new gui.MenuItem("Save settings", main.system.saveSettings_onClick) );
main.system.append( new gui.MenuItem("Reset defaults", main.system.factoryDefault_onClick) );
/*****************************************************************************/



notification = new gui.Text("");
notification.leftSoftkey = new gui.Softkey("Ok",  function() { gui.showMenu(main); });



notification.show = function(text)
{
    this.text = text? text: "Ready";
    gui.showDialog(this);
}



var lastCmdType = "";
var lastCmdData = "";



reader.onCommand = function(type, data)
{
    lastCmdType = type;
    lastCmdData = data;

    notification.show("processing command: '" + lastCmdType + ':' + lastCmdData + "'...");
    return true;
}



reader.onCommandFinish = function(ok, type, data)
{
    notification.show( "command "
        + ( ok? "successful"
                : ("'" + lastCmdType + ':' + lastCmdData + "' failed: " + type + ':' + data)
            )
        );
}



reader.onDecode = function(decode)
{
    notification.show("Decode ignored, scan again after done with configuration");
    return null;
}



/*****************************************************************************/
// Send/Log Submenu
sendAndStoreOptions = new gui.Menu();
sendAndStoreOptions.caption = "Send/Log";
sendAndStoreOptions.changed = false;

sendAndStoreOptions.autoSend = new gui.ToggleButton
    ( "Always send"
    , true
    , function(checked, text)
      {
          var i = reader.readSetting(setting_sendStoreMode);

          if( checked )
              i &= ~noAutoSendBit;
          else
              i |=  noAutoSendBit;

          reader.writeSetting(setting_sendStoreMode, i);
          sendAndStoreOptions.changed = true;
      }
    );



sendAndStoreOptions.alwaysStore = new gui.ToggleButton
    ( "Always log"
    , false
    , function(checked, text)
      {
          var i = reader.readSetting(setting_sendStoreMode);

          if( checked )
              i |=  alwaysStoreBit;
          else
              i &= ~alwaysStoreBit;

          reader.writeSetting(setting_sendStoreMode, i);
          sendAndStoreOptions.changed = true;
      }
    );



sendAndStoreOptions.autoErase = new gui.ToggleButton
    ( "Auto erase"
    , true
    , function(checked, text)
      {
          reader.writeSetting(setting_autoLogErase, checked? 1: 0);
          sendAndStoreOptions.changed = true;
      }
    );



sendAndStoreOptions.autoUpload = new gui.ToggleButton
    ( "Auto upload"
    , true
    , function(checked, text)
      {
          reader.writeSetting(setting_autoBufferUpload, checked? 1: 0);
          sendAndStoreOptions.changed = true;
      }
    );



sendAndStoreOptions.append(sendAndStoreOptions.autoSend);
sendAndStoreOptions.append(sendAndStoreOptions.alwaysStore);
sendAndStoreOptions.append(sendAndStoreOptions.autoErase);
sendAndStoreOptions.append(sendAndStoreOptions.autoUpload);



sendAndStoreOptions.saveOnBack = function()
{
    gui.confirm(sendAndStoreOptions.confirmSaveOnBack, null, "Settings changed.\n\nSave now?");
}



sendAndStoreOptions.confirmSaveOnBack = function()
{
	reader.saveSettings();
	reader.runScript(".default.js");
}



sendAndStoreOptions.onKey = function(key)
{
    if( key == gui.key.back  ||  key == gui.key.escape )
    {
		if( this.changed)
        {
            sendAndStoreOptions.saveOnBack();
        }
    }
}



sendAndStoreOptions.update = function()
{
    sendAndStoreOptions.autoSend   .check(!(reader.readSetting(setting_sendStoreMode)    &  noAutoSendBit));
    sendAndStoreOptions.alwaysStore.check(  reader.readSetting(setting_sendStoreMode)    &  alwaysStoreBit);
    sendAndStoreOptions.autoErase  .check(  reader.readSetting(setting_autoLogErase)     != 0);
    sendAndStoreOptions.autoUpload .check(  reader.readSetting(setting_autoBufferUpload) != 0);

    this.changed = false;

    gui.showSubMenu(this, main);
}
/*****************************************************************************/



// main
main.onCancel = function()
{
    reader.runScript(".default.js");
}



main.caption = "Configuration";
main.append( new gui.MenuItem("Application",     function() { gui.showSubMenu(main.app, main); }) );
main.append( new gui.MenuItem("Communication",   main.comm.interfaces_onClick) );
main.append( new gui.MenuItem("Send/Log",        function() { sendAndStoreOptions.update(); }) );
main.append( new gui.MenuItem("Reader Info",     function() { gui.showSubMenu(main.info, main); }) );
main.append( new gui.MenuItem("System",          function() { gui.showSubMenu(main.system, main); }) );

gui.showMenu(main);



// EOF
