/* .cre.js
 * $Revision: 11135 $
 *
 * default CR8000/CR1000 application code
 * sends/stores and displays decode data; uploads and clears on request
 * provides link to configuration application
 *
 * Copyright 2011 The Code Corporation. All rights are reserved.
 *
 */


///////////////////////////////////////////////////////////////
// include statements

include(".settings.js");



var rules_onDecodeAttempt = null;
var rules_onDecode        = null;
var rules_onEvent         = null;

var rulesFile = storage.findFirst(/^\.rules.*\.js$/);
if( rulesFile != null )
    include(rulesFile);



///////////////////////////////////////////////////////////////
// constants

sepHeight                      =    3;

noStorage                      =    0;
someStorage                    =   25; /*percent*/
fullStorage                    =  100; /*percent*/

errorBeep                      =    3;

autoConnectIfCabled            =    1;
autoConnectIfNeedToSend        =    2;

connectionPollingPeriod_ms     =  500;

secondsInYear = 31536000;
secondsInDay = 86400;
secondsInHour = 3600;

codeXMLStart1 = '\001X\036be/';
codeXMLStart2 = '\001X\036bf/';
codeXMLEnd   = '\004';

decoder_RSS_Expanded          = 43;
decoder_RSS_Expanded_Stacked  = 44;
decoder_RSS_Limited           = 45;
decoder_RSS_14                = 46;
decoder_RSS_14_Stacked        = 47;


comm_sendFailed = 0;
comm_sendPending = 1;
comm_sendSuccess = 2;
comm_sendBusy = 3;

///////////////////////////////////////////////////////////////
// messages
//TODO: bring all messages here (for translation and such)
//   parameters: interactive/quiet, ready/busy, info/error/progress



///////////////////////////////////////////////////////////////
// manual entry form

manual         = new gui.Form();
manual.caption = "Manual Entry";
manual.edit    = new gui.Edit("");
manual.append(manual.edit);


///////////////////////////////////////////////////////////////
// post Alert function to show default form
runDefault = function()
{
	ready.show();
}


manual.onOk = function()
{
    if( manual.edit.text == "" )
        ready.show();
    else
    {
        store.processData(manual.edit.text);
        manual.edit.text = "";
    }
};



manual.onCancel = function()
{
    manual.edit.text = "";
    ready.show();
};



///////////////////////////////////////////////////////////////
// storage dialog

store = new gui.Text("");

store.leftSoftkey  = new gui.Softkey("Options", function() { gui.showMenu(store.options); });
store.rightSoftkey = new gui.Softkey("Send",    function() { store.connectAndSend(); });

store.first        = 9999999;
store.next         = 0;
store.current      = 0;
store.index        = 0;
store.count        = 0;
store.sendPending  = false;


store.checkSettings = function()
{
    this.autoSend         = !(reader.readSetting(setting_sendStoreMode)    &  noAutoSendBit);
    this.alwaysStore      =   reader.readSetting(setting_sendStoreMode)    &  alwaysStoreBit;
    this.autoErase        =   reader.readSetting(setting_autoLogErase)     != 0;
    this.autoBufferUpload =   reader.readSetting(setting_autoBufferUpload) != 0;
    this.maxBufferQueueSize  =   reader.readSetting(setting_maxBufferedQueueSize);
}



store.updatePrefixSuffix = function()
{
    this.prefix = storage.read(".prefix");
    if( !this.prefix )
        this.prefix = "";

    this.suffix = storage.read(".suffix");
    if( !this.suffix )
        this.suffix = "";
}



store.init = function()
{
    var prefixLength = 4; /* length of data filename prefix, either ".buf" or ".log" */

    store.checkSettings();

    this.bufferCount = 0;
    this.sendPending = false;
    this.bufferDataArray = new Array();

    for(var name = storage.findFirst( /^\.(log)|(buf)\d+$/ );
        name;
        name = storage.findNext()
       )
    {
        var id = parseInt(name.substr(prefixLength));

        if( id < this.first )
            this.first = id;

        if( id >= this.next )
            this.next = id + 1;

        ++this.count;

        if( name.substr(0, prefixLength) == ".buf" )
            ++this.bufferCount;
    }

    if( this.count == 0 )
        this.first = this.next = 1000000;
    else
        gui.indicateMemory(this.count > this.bufferCount, this.bufferCount > 0);

    this.current = this.first;

	//Indicate Comm Mode during intialization.
	gui.indicateCommMode();

    store.updatePrefixSuffix();
}



store.readUp = function()
{
    var data;

    for(; this.current < this.next; ++this.current )
    {
        if(      (data = storage.read(".log" + this.current)) )
        {
            this.buffered = false;
            break;
        }
        else if( (data = storage.read(".buf" + this.current)) )
        {
            this.buffered = true;
            break;
        }
    }

    return data;
}



store.readDown = function()
{
    var data;

    for(; this.current >= this.first; --this.current )
    {
        if(      (data = storage.read(".log" + this.current)) )
        {
            this.buffered = false;
            break;
        }
        else if( (data = storage.read(".buf" + this.current)) )
        {
            this.buffered = true;
            break;
        }
    }

    return data;
}



store.readCurrent = function()
{
    var data;

    if(      (data = storage.read(".log" + this.current)) )
        this.buffered = false;
    else if( (data = storage.read(".buf" + this.current)) )
        this.buffered = true;

    return data;
}



store.showData = function(data, showMessageIfEmpty)
{
    if( this.count == 0  ||  this.index >= this.count )
    {
        if( !this.quiet )
            ready.show(showMessageIfEmpty? "No stored data": null);

        gui.indicateMemory(false, false);
    }
    else
    {
        this.text = data? data: this.readCurrent();

        this.caption = (this.buffered? "*": "") + (this.index + 1) + " of " + this.count;

        if( !this.quiet )
            gui.showDialog(this);
    }
}



store.showFirst = function(showMessageIfEmpty)
{
    this.index   = 0;
    this.current = this.first;
    this.showData(this.readUp(), showMessageIfEmpty);
}



store.showLast = function()
{
    this.index   = this.count == 0? 0: this.count - 1;
    this.current = this.next;
    this.showData(this.readDown());
}



store.showNext = function()
{
    ++this.index;
    ++this.current;
    if( this.index == this.count )
        this.showFirst();
    else
        this.showData(this.readUp());
}



store.showPrev = function()
{
    --this.index;
    --this.current;
    if( this.index < 0 )
        this.showLast();
    else
        this.showData(this.readDown());
}



store.onKey = function(key)
{
    switch( key )
    {
    case gui.key.enter:
        this.edit();
        break;
    case gui.key.back:
    case gui.key.escape:
        ready.show();
        break;
    case gui.key.left:
        this.showPrev();
        break;
    case gui.key.right:
        this.showNext();
        break
    default:
        gui.showForm(manual);
        if( key != gui.key.enter )
            gui.sendKey(key);
    }
};

store.indicateStorageFailure = function()
{
    // Indicate failure to store file (beep and flash data LED)
    if (reader.blinkDataLedIntervalId == -1)
    {
        reader.blinkDataLedCount = 10;
        reader.blinkDataLed();
        reader.blinkDataLedIntervalId = setInterval(reader.blinkDataLed, 100);
        reader.processCommand('#', '\3'); // error beep
    }
}

store.indicateWirelessSendFailure = function()
{
    // Indicate failure to send data wirelessly
    if (reader.blinkWirelessLedIntervalId == -1)
    {
        reader.blinkWirelessLedCount = 10;
        reader.blinkWirelessLed();
        reader.blinkWirelessLedIntervalId = setInterval(reader.blinkWirelessLed, 100);
    }
}

store.store = function(data, buffered)
{
    var name = (buffered? ".buf": ".log") + this.next;

    if( storage.write(name, data) )
    {
        ++this.next;
        ++this.count;

        if( buffered )
            ++this.bufferCount;

        gui.indicateMemory(this.count > this.bufferCount, this.bufferCount > 0);

        return true;
    }

    store.indicateStorageFailure();
    return false;
}

store.sendBuffered = function(outData)
{
    // This could call sendPacketBuffered() or sendPacket() based on enableBackgroundTransmit setting
    // or downlevel firmware
    var sentStatus = comm.sendPacketBuffered('z', outData);

    if ((sentStatus == comm_sendPending) || (sentStatus == comm_sendBusy))
    {
        // Background transmit mode
        store.sendPending = true;
    }
    else
    {
        // Non-background transmit -- legacy mode
        var needStore = store.alwaysStore || !store.autoSend;

        // take data off buffer
        this.bufferDataArray.shift();

        if (sentStatus == comm_sendFailed)
        {
            if (reader.readSetting(setting_noStoreIfNotConnected))
            {
                if( commMode_bt == reader.readSetting(setting_commMode) )
                {
                    store.indicateWirelessSendFailure();
                }
                reader.beep(errorBeep);
            }
            else
            {
                needStore = true;
            }
        }

        if (needStore)
            storedOk = this.store(outData, store.autoSend && (sentStatus == comm_sendFailed));
    }
    return sentStatus;
}

// This function is used to convert ram files into .buf files
store.dumpRamDataToFile = function()
{
    while( this.bufferDataArray.length > 0 )
    {
        var tempData = this.bufferDataArray.shift();
        storedOk = store.store(tempData, true);
    }
}

store.bufferedDataProcess = function(activeDecode)
{
    var needPriorDataFileStore = false;
    var needErrorBeep          = false;
    var priorSendData          = "";
    var lastSendStatus  = comm_sendSuccess;
    var sentDataOk      = false;
    var noStore         = reader.readSetting(setting_noStoreIfNotConnected);
    var currentData     = "";
    var updatedStatus   = false;

    storedOk        = false;

    if (activeDecode)
        this.bufferDataArray.push(store.prefix + store.data + store.suffix);

    var isConnected = comm.isConnected;

    if (store.sendPending)
    {
        // STATE -- Prior send pending, need to check status of last send
        lastSendStatus = comm.getSendStatus();

        if(this.bufferDataArray.length > this.maxBufferQueueSize )
        {
            // Block here
            while( lastSendStatus == comm_sendBusy)
            {
                lastSendStatus = comm.getSendStatus()
            }
        }

        if ((lastSendStatus == comm_sendFailed) || (lastSendStatus == comm_sendSuccess))
        {
            priorSendData = this.bufferDataArray.shift();
            store.sendPending = false;
            needPriorDataFileStore = (store.alwaysStore || !store.autoSend);

            if (lastSendStatus == comm_sendFailed)
            {
                if (!noStore)
                {
                    needPriorDataFileStore = true;
                }
                else
                {
                    needErrorBeep = true;

                    // If we are failing and not storing data, flush queue to avoid
                    // and possibility that we could send data out of order
                    while( this.bufferDataArray.length > 0 )
                    {
                        this.bufferDataArray.shift();
                    }
                }
                sentDataOk = false;
            }
            else
            {
                sentDataOk = true;
            }

            if (needPriorDataFileStore)
            {
                storedOk = this.store(priorSendData, store.autoSend && (lastSendStatus == comm_sendFailed));
            }
            currentData = priorSendData;
            updatedStatus = true;
        }
    }

    if (!(store.sendPending) && (store.bufferCount > 0) && (store.autoBufferUpload) && isConnected)
    {
        // STATE -- Have file buffered data
        store.sendAll(true);
    }

    if (!(store.sendPending) && (this.bufferDataArray.length > 0))
    {
        // STATE -- Have RAM buffered data
        currentData = this.bufferDataArray[0];
        if( isConnected )
        {
            if (store.bufferCount == 0)
            {
                if (this.autoSend)
                {
                    var currentSendStatus = store.sendBuffered(this.bufferDataArray[0]);
                    if( (currentSendStatus == comm_sendFailed) || (currentSendStatus == comm_sendSuccess) )
                    {
                        updatedStatus = true;
                        if( currentSendStatus == comm_sendFailed )
                        {
                            sentDataOk = false;
                        }
                        else
                        {
                            sentDataOk = true;
                        }
                    }
                }
                else if (this.alwaysStore)  // no send, but create log file here
                {
                    updatedStatus = true;
                    currentData = this.bufferDataArray[0];
                    storedOk = this.store(this.bufferDataArray.shift(), false);
                }
            }
        }
        else
        {
            updatedStatus = true;
            // Not connected
            if( noStore )
            {
                // Flush ram queue to avoid patent problems with buffering while not connected
                while( this.bufferDataArray.length > 0 )
                {
                    this.bufferDataArray.shift();
                }

                needErrorBeep = true;
            }
            else
            {
                store.dumpRamDataToFile();
            }
        }
    }

    if( needErrorBeep )
    {
        if( commMode_bt == reader.readSetting(setting_commMode) )
        {
            store.indicateWirelessSendFailure();
        }

        reader.beep(errorBeep);
    }

    store.disconnect();

    if( reader.hasDisplay )
    {
        if( updatedStatus )
        {
            var status = sentDataOk &&  storedOk               ? "Sent and stored"
                :        sentDataOk && !needPriorDataFileStore ? "Sent"
                :        sentDataOk                            ? "Sent BUT NOT STORED"
                :        storedOk                              ? "Stored"
                :                                                "NOT SENT OR STORED"
                ;

            ready.show(status + ":\n" + currentData);
        }
    }

    store.quiet = false;
}


store.waitForConnection = function()
{
    var noStore         = reader.readSetting(setting_noStoreIfNotConnected);
    var synchronousSend = reader.readSetting(setting_synchronousSend);

    if( !comm.isConnected && noStore && synchronousSend && store.duration_ms >= 0 )
    {
        // TBD -- Background connect doesn't work in CR2600 yet
        store.duration_ms -= connectionPollingPeriod_ms;
        setTimeout(store.waitForConnection, connectionPollingPeriod_ms);
    }
    else
    {
        // We have active decode data, pass in true
        store.bufferedDataProcess(true);
    }
}



store.processData = function (data)
{
    this.quiet = true;

    ready.show("Processing data:\n", true);

    this.connect(reader.readSetting(setting_autoConnectMode) & autoConnectIfNeedToSend);
    this.duration_ms = reader.readSetting(setting_implicitConnectTimeout_ms);
    this.data = data;
    store.waitForConnection();
}

store.formatNumberLength = function(num, length)
{
    var r = "" + num;
    while (r.length < length)
    {
        r = "0" + r;
    }
    return r;
}

store.calculateDateAndTime = function(seconds)
{
    // Roughly calculate time
    var daysInMonthArray = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    var years;
    var months;
    var days;
    var hours;
    var mins;

    // Years
    years = Math.floor(seconds/secondsInYear);

    var remainingSecs = seconds - years*secondsInYear;
    var countdown = remainingSecs;

    // Month
    for(var i = 0; i < 12; ++i)
    {
        countdown -= (daysInMonthArray[i] * secondsInDay);

        if (countdown < 0)
        {
            months = i;
            countdown += (daysInMonthArray[i] * secondsInDay);
            break;
        }
    }

    // Days
    days = Math.floor(countdown/secondsInDay);

    remainingSecs = countdown - (days*secondsInDay);

    // Hours
    hours = Math.floor(remainingSecs/secondsInHour);

    remainingSecs = remainingSecs - (hours*secondsInHour);

    // Minutes
    mins = Math.floor(remainingSecs/60);

    // Seconds
    remainingSecs = remainingSecs - (mins*60);

    months = this.formatNumberLength(months + 1, 2);
    days = this.formatNumberLength(days + 1, 2);
    hours = this.formatNumberLength(hours, 2);
    mins = this.formatNumberLength(mins, 2);
    remainingSecs = this.formatNumberLength(remainingSecs, 2);

    var dateTimeString =    (years + 1970).toString() + "-" +
                            months.toString() + "-" +
                            days.toString() + " " +
                            hours.toString() + ":" +
                            mins.toString() + ":" +
                            remainingSecs.toString();
    return(dateTimeString);
}

store.preprocessData = function(decode)
{

    var decodeExtras = reader.readSetting(setting_decodingExtras);

    if (decodeExtras)
    {
        var decodeSymbology = decode.symbology;

        if((decodeExtras & removeGS1DatabarParens) &&
           ((decodeSymbology >= decoder_RSS_Expanded) && (decodeSymbology <= decoder_RSS_14_Stacked)))
        {
            var parenIndex = decode.data.indexOf('(');
            if (parenIndex >= 0)
            {
                decode.data = decode.data.substring(0,parenIndex) + decode.data.substring(parenIndex + 1);
            }

            parenIndex = decode.data.indexOf(')');
            if (parenIndex >= 0)
            {
                decode.data = decode.data.substring(0,parenIndex) + decode.data.substring(parenIndex + 1);
            }
        }

        if(decodeExtras & forceUpperCase)
        {
            decode.data = decode.data.toUpperCase();
        }
        else if (decodeExtras & forceLowerCase)
        {
            decode.data = decode.data.toLowerCase();
        }

    }

    // Add on any additional prefix information (not related to prefix file, if present)
    // prefixWithTimestamp
    if (reader.readSetting(setting_prefixWithTimestamp))
    {
        var now = new Date();
        decode.data = this.calculateDateAndTime(now.getTime()/1000) + decode.data;
    }

    // prefixWithQuality
    if (reader.readSetting(setting_prefixWithQuality))
    {
        decode.data = codeXMLStart1 + decode.quality_percent +  codeXMLEnd + decode.data;
    }

    //  prefixWithSymbology
    var prefixSymb = reader.readSetting(setting_prefixWithSymbology);
    if (prefixSymb == symbologyPrefix)
    {
        decode.data = codeXMLStart2 + decode.symbology +  codeXMLEnd + decode.data;
    }
    else if (prefixSymb == symbologyAndModifierPrefix)
    {
        decode.data = codeXMLStart2 + decode.symbology +  '-' + decode.symbologyModifier + codeXMLEnd + decode.data;
    }

    // enableSymbologyIdentifierPrefix
    if (reader.readSetting(setting_enableSymbologyIdentifierPrefix))
    {
        decode.data = ']' + String.fromCharCode(decode.aimSymbology) +
                            String.fromCharCode(decode.aimModifier) +
                            decode.data;
    }
}


store.erase = function()
{
    if( !this.quiet )
        gui.statusText = "Deleting...";

    if( !storage.erase(".log" + this.current) )
    {
        storage.erase(".buf" + this.current);
        --this.bufferCount;
    }

    --this.count;

    if( this.count > 0  &&  this.index >= this.count )
    {
        --this.index;
        --this.current;
        this.showData(this.readDown());
    }
    else
    {
        this.showData(this.readUp());

        if( this.index == 0 )
        {
            this.first = this.current;
        }
    }
}



store.eraseAll = function()
{
    ready.show("Deleting...", true);

    for(var i = this.first; i < this.next; ++i)
    {
        if( !storage.erase(".log" + i) )
            storage.erase( ".buf" + i);
    }

    this.count       = 0;
    this.bufferCount = 0;
    this.index       = 0;
    this.current     = this.first = this.next - 1;

    gui.indicateMemory(false, false);

    ready.show();
}



store.send = function()
{
    if( !this.quiet )
        gui.statusText = "Sending...";

    var sentOk = comm.sendPacket('z', this.text);

    if( sentOk )
    {
        if( store.autoErase )
            this.erase();
        else
        {
            if( this.buffered )
            {
                storage.rename(".buf" + this.current, ".log" + this.current);
                this.buffered = false;
                --this.bufferCount;
            }

            gui.statusText = '#' + (this.index + 1) + " Sent";
        }
    }
    else if( !this.quiet )
    {
        gui.statusText = (this.buffered? "*": "") + (this.index + 1) + " of " + this.count;
        gui.alert(runDefault, "Send failed");
    }

    return sentOk;
}



store.sendAll = function(onlyBufferedItems)
{
    var ok  = true;
    var raw = reader.readSetting(setting_commProtocol) == commProtocolRaw;

    if( this.count == 0  ||  onlyBufferedItems && this.bufferCount == 0 )
    {
        if( !this.quiet )
            ready.show(onlyBufferedItems? "No buffer data to send": "Nothing to send");

        return true;
    }

    this.index   = 0;
    this.current = this.first;
    this.showData(this.readUp());

    if( !this.quiet )
        ready.show("Sending...", true);

    if( !raw )
    {
        if( !comm.sendPacket('g', onlyBufferedItems? "set of buffered codes": "set of all stored codes") )
        {
            if( !this.quiet )
            {
                gui.alert(runDefault, "Send failed");
            }

            return false;
        }
    }

    while( this.index < this.count )
    {
        var skip = onlyBufferedItems && !this.buffered;

        if( !skip )
        {
            if( !this.send() )
            {
                ok = false;
                break;
            }

            if( !this.autoErase )
                skip = true;
        }

        if( skip )
        {
             ++this.index;
             ++this.current;
             this.showData(this.readUp());
        }
    }

    if( !raw )
        comm.sendPacket(ok? 'd': 'e', ok? "complete set": "incomplete set");

    if( !this.quiet )
        ready.show(ok? (onlyBufferedItems? "Sent all buffered codes": "Sent all"): "Send failed");

    return ok;
}



store.connect = function(force)
{
    var connected;
    var now = new Date();
    var sec = now.getTime() / 1000/*ms/sec*/;

    if(     force
        ||  (   (       reader.readSetting(setting_autoConnectMode) == autoConnect
                ||  (   (reader.readSetting(setting_autoConnectMode) & autoConnectIfCabled) &&  reader.cabled )
                ||  (   reader.readSetting(setting_autoConnectMode) == autoReconnect
                    &&  sec - this.lastConnectedTime_sec < reader.readSetting(setting_reconnectTimeout_sec)
                    )
                )
            &&  (     reader.readSetting(setting_autoDisconnect) == 0
                ||  ( this.autoBufferUpload  &&  this.bufferCount > 0 )
                )
            )
      )
    {
        var show = force && !this.quiet && !comm.isConnected;

        if( show )
            ready.show("Connecting...", true);

        connected = comm.connect(force);

        if( show )
            ready.show();
    }
    else
    {
        connected = comm.isConnected;
    }

    if( connected )
        this.lastConnectedTime_sec = sec;

    return connected;
}



store.disconnect = function(force)
{
    if(    force
        || (     reader.readSetting(setting_autoDisconnect)  != 0
           && ( !this.autoBufferUpload  ||  this.bufferCount == 0 )
           )
      )
    {
        comm.disconnect();
    }
}



store.connectAndSend = function()
{
    var ok = false;

    var connected = this.connect(true);

    this.showData();

    if( connected )
    {
        this.showData();

        ok = this.send();

        this.disconnect();
    }
    else if( !this.quiet )
    {
        gui.alert(runDefault, "Connect failed");
    }

    return ok;
}



store.connectAndSendAll = function(onlyBufferedItems, forceConnectionIfNeeded)
{
    var needSend = (onlyBufferedItems? this.bufferCount: this.count) > 0;
    var force    = forceConnectionIfNeeded && needSend;
    var ok       = false;

    if( this.connect(force) )
    {
        ok = this.sendAll(onlyBufferedItems);
        this.disconnect();
    }
    else if( !this.quiet )
    {
        if( needSend )
            gui.alert(runDefault, "Connect failed");
        else
            ready.show(onlyBufferedItems? "No buffer data to send": "Nothing to send");
    }

    return ok;
}



store.edit = function()
{
    var ok = true;
    var s;

    if( (s = gui.prompt(null, "Edit data", this.text, gui.statusText)) )
    {
        var suffixReg = new RegExp  ("^.*"+this.suffix+"$");
        if(s.match(suffixReg)==null)
            s=s+this.suffix

        var prefixReg = new RegExp ("^"+this.prefix+".*$");
        if(s.match(prefixReg)==null)
            s=this.prefix+s

        ok = storage.write((this.buffered? ".buf": ".log") + this.current, s);
    }

    this.showData(this.readCurrent());

    if( !ok )
        reader.processCommand('#', '\3');

    return ok;
}



store.viewAsHex = function()
{
    var ok = true;
    var s  = "";

    for(var i = 0; i < this.text.length; ++i)
    {
        s = s + format("%02x ", this.text.charCodeAt(i));
    }

	 gui.alert(null, s, "Hex View");
}



// storage options dialog

store.options = new gui.Menu();
store.options.caption = "Options";

store.options.deleteMenuItem = function()
{
    gui.confirm( store.options.confirmDeleteMenuItem, null, "Permanently delete this record?");
}

store.options.confirmDeleteMenuItem = function()
{
    store.erase();
}

store.options.deleteAllItems = function()
{
    gui.confirm(store.options.confirmDeleteAllItems, null, "Permanently delete all records?");
}

store.options.confirmDeleteAllItems = function()
{
	store.eraseAll();
}

store.options.append( new gui.MenuItem("Edit",        function() { store.edit(); }) );
//store.options.append( new gui.MenuItem("View as hex", function() { store.viewAsHex(); }) );
store.options.append( new gui.MenuItem("Send",        function() { store.connectAndSend(); }) );
store.options.append( new gui.MenuItem("Delete",      store.options.deleteMenuItem) );
store.options.append( new gui.Separator(sepHeight, gui.separatorStyle.horizontalLine) );
store.options.append( new gui.MenuItem("Send all",    function() { store.connectAndSendAll(false, true); }) );
store.options.append( new gui.MenuItem("Delete all",  store.options.deleteAllItems) );
store.options.append( new gui.Separator(sepHeight, gui.separatorStyle.horizontalLine) );
store.options.append( new gui.MenuItem("Enter data", function() { gui.showForm(manual); }) );
if( !reader.readSetting(setting_disableConfigMenu) )
{
    store.options.append( new gui.Separator(sepHeight, gui.separatorStyle.horizontalLine) );
    store.options.append( new gui.MenuItem("Settings", function() { settings(); }) );
}
store.options.onCancel = function() { gui.showDialog(store) };



///////////////////////////////////////////////////////////////
// settings (link to config application)

function settings()
{
    ready.show("Loading...", true);
    reader.runScript(".configure.js");
}



///////////////////////////////////////////////////////////////
// ready dialog

ready = new gui.Text("");



ready.onKey = function(key)
{
    switch( key )
    {
    case gui.key.left:
        store.showLast();
        break;
    case gui.key.right:
        store.showFirst();
        break
    case gui.key.back:
    case gui.key.escape:
        this.show();
        break;
    default:
        gui.showForm(manual);
        if( key != gui.key.enter )
            gui.sendKey(key);
    }
};



ready.show = function(text, busy)
{
    this.text = text? (busy? text: ("Ready\n(" + text + ")")): "Ready";
    gui.showDialog(this);
}



// options menu

ready.options           = new gui.Menu();
ready.options.caption   = "Options";

ready.options.append( new gui.MenuItem("Enter data",   function() { gui.showForm(manual); }) );
ready.options.append( new gui.MenuItem("View storage", function() { store.showFirst(true); }) );
ready.options.append( new gui.MenuItem("Connect",      function() { ready.show(store.connect(true)? "Connected": "Connect failed"); }) );
ready.options.append( new gui.MenuItem("Send all",     function() { store.connectAndSendAll(false, true); }) );

if( !reader.readSetting(setting_disableConfigMenu) )
{
    ready.options.append( new gui.Separator(sepHeight, gui.separatorStyle.horizontalLine) );
    ready.options.append( new gui.MenuItem("Settings", function() { settings(); }) );
}

ready.options.onCancel = function() { ready.show() };

ready.leftSoftkey = new gui.Softkey("Options",  function() { gui.showMenu(ready.options); });



///////////////////////////////////////////////////////////////
// onDecode etc.

reader.onDecodeAttempt = function(count)
{
    var result = true;

    if( rules_onDecodeAttempt != null )
        result = rules_onDecodeAttempt(count);

    if( !result )
        return false;

    if( count == 0 )
        ready.show();
}



reader.onDecode = function(decode)
{
    if( rules_onDecode != null )
        decode = rules_onDecode(decode);

    if( !decode )
        return false;

    store.preprocessData(decode);

    store.processData(decode.data);
}


function listFiles(re, dirNum)
{
    var appArray = new Array();
    var app;

    comm.sendPacket('g', "file-list\x00");

    if( dirNum == currDir )
    {
        app = storage.findFirst(re);
    }
    else if( dirNum == decoderDir )
    {
        app = storage.findFirstIn(re, decoderDir);
    }

    if ( app != null )
    {
        appArray[0] = app;

        var i = 1;
        while ( (app = storage.findNext()) != null )
        {
            appArray[i] = app;
            i = i + 1;
        }

        appArray.sort();

        for ( var k = 0; k < appArray.length; k++ )
            comm.sendPacket('z', appArray[k] + "\x00");
    }
    comm.sendPacket('d', "EOF\x00");
}

function showRevisions()
{
    var appArray = new Array();
    var re = new RegExp("\\$" + "Revision: .*\\$"); // separate $ and Version to avoid RCS keyword substitution
    var app = storage.findFirst(/^.+\.js$/);

    if ( app != null )
    {
        var i = 0;
        do
        {
            var file = storage.getHeader(app);

            if( file != null )
            {
                var ret = file.match(re);

                if ( ret == null )
                {
                    ret = " N/A";
                }
                else
                {
                    ret = ret.toString();
                    ret = ret.replace(/\$|Revision: /g, "");
                    ret = " "+ ret;
                }
                appArray[i] = app + ret;
                i = i + 1;
            }
            else
            {
                ret = " N/A";
                appArray[i] = app + ret;
                i = i + 1;
            }
        }
        while ( (app = storage.findNext()) != null );

        appArray.sort();

        for ( var k = 0; k < appArray.length; k++ )
            comm.sendPacket('z', appArray[k]);
    }
}



function erasePictures()
{
    while( name = storage.findFirst(/^picture[0-9]+\.((jpg)|(pgm))$/) )
        storage.erase(name);
}


function deleteRules()
{
    while( name = storage.findFirst(/^\.rules.*\.js$/) )
        storage.erase(name);
}


var lastCmdType       = "";
var lastCmdData       = "";
var runCmd            = null;

reader.onCommand = function(type, data)
{
    if( type == '|'  &&  data.match(/^run\:.*\.js$/)  &&  !reader.readSetting(setting_disableRunCommand) )
    {
        runCmd = data.substr(4/*len of "run:"*/);
        ready.show(data + "...", true);

        return false;
    }
    else
    {
        lastCmdType = type;
        lastCmdData = data;

        if(      type == '1' )
            ready.show("downloading...", true);
        else if( type == '5' && data.match(/^\.rules.*\.js\x00$/) )
            deleteRules();
        else if( type == '9' && data.match(/^\.rules\*\.js\x00?$/) ) //match null terminated or not (like firmware does)
        {
            deleteRules();
            return false;
        }
        else if( type == '?' )
            ready.show("processing license...", true);
        else if( type == 'L' )
        {
            return false;
        }
        else
            ready.show("processing command: '" + lastCmdType + ':' + lastCmdData  + "'...", true);

        return true;
    }
}

reader.onCommandFinish = function(ok, type, data)
{
    if( runCmd )
    {
        if (reader.runScript(runCmd))
            comm.sendPacket('d', "run script");
        else
            comm.sendPacket('e', "cannot run script");
        runCmd = null;
    }
    else if( lastCmdType == 'N' )
    {
        store.eraseAll();
        erasePictures();
    }
    else if( lastCmdType == 'L')
    {
        if( lastCmdData == '0' || lastCmdData == "")
        {
            listFiles(/^[^\.]/, currDir);
        }
        else if( lastCmdData == '1' )
        {
            listFiles(/^./, currDir);
        }
        else if( lastCmdData == '2' )
        {
            showRevisions();
        }
        else if( lastCmdData == 'd' )
        {
            listFiles(/^./, decoderDir);
        }

        lastCmdData = "";
        lastCmdType = "";
    }
    else if( lastCmdType == 'X' )
    {
        var onlyBufferedItems = lastCmdData != "1";

        if( (onlyBufferedItems? store.bufferCount: store.count) == 0 )
        {
            ready.show(onlyBufferedItems? "No buffer data to send": "Nothing to send");
        }
        else
        {
            store.quiet = true;
            // TBD -- When is the 'X' Command type issued
            ok = store.connectAndSendAll(onlyBufferedItems, true);

            store.quiet = false;

            ready.show(ok? (onlyBufferedItems? "Sent all buffered codes": "Sent all"): "Send failed");

            if( !ok )
                reader.beep(errorBeep);
        }
    }
    else if( lastCmdType == '?' )
    {
        ready.show("License install" + (ok? "ed": " failed"));
    }
    else if( lastCmdType != '1' )
    {
        if( lastCmdType.match(/[\/OPQJC]/) )
        {
            store.checkSettings();
        }

        ready.show( "Command "
            + ( ok? "successful"
                  : ("'" + lastCmdType + ':' + lastCmdData + "' failed: " + type + ':' + data)
              )
            );
    }
}



var prevRfActiveConnection = reader.readSetting(setting_activeRfConnection);
var prevBtKBActiveConnection = reader.readSetting(setting_activeBtKbConnection);

reader.onIdle = function()
{
    var buffered = store.bufferCount > 0;

    store.quiet = true;

    if(    reader.readSetting(setting_rfAutoSaveActiveConnection)
        && comm.isConnected
      )
    {
        // Now check comm mode
        if (commMode_bt == reader.readSetting(setting_commMode)
            && reader.readSetting(setting_activeRfConnection) != prevRfActiveConnection)
        {
            reader.processCommand("=", String.fromCharCode(setting_activeRfConnection) + "#" + reader.readSetting(setting_activeRfConnection));
            prevRfActiveConnection = reader.readSetting(setting_activeRfConnection);
        }
        else if (commMode_btKb == reader.readSetting(setting_commMode)
            && reader.readSetting(setting_activeBtKbConnection) != prevBtKBActiveConnection)
        {
            // Do this slightly different than commMode_bt -- String.fromCharCode doesn't working with larger setting numbers
            reader.processCommand("=", "(" + setting_activeBtKbConnection.toString(16) + ")#" + reader.readSetting(setting_activeBtKbConnection));
            prevBtKBActiveConnection = reader.readSetting(setting_activeBtKbConnection);
        }
    }

    store.connect(false);
    store.quiet = false;

    store.bufferedDataProcess(false);

    if( buffered  &&  store.bufferCount == 0 )
        ready.show("Sent all buffered data");
}

reader.onUpdate = function()
{
    // Do a quick check to see if there is pending data (non-volatile or volatile)
    //  that needs to be sent or status checked of prior send
    if((store.sendPending) ||
       ((store.bufferCount > 0) && (store.autoBufferUpload)) ||
       (store.bufferDataArray.length > 0))
    {
        store.bufferedDataProcess(false);
    }
}


reader.blinkDataLedIntervalId = -1;
reader.blinkDataLedCount = 0;
reader.blinkDataLed = function()
{
    // Blink the data LED
    if (reader.blinkDataLedCount > 0)
        reader.blinkDataLedCount--;
    reader.indicateDataLed(reader.blinkDataLedCount % 2);
    if (reader.blinkDataLedCount <= 0)
    {
        if (reader.blinkDataLedIntervalId >= 0)
        {
            clearInterval(reader.blinkDataLedIntervalId);
            reader.blinkDataLedIntervalId = -1;
        }
    }
}

reader.blinkWirelessLedIntervalId = -1;
reader.blinkWirelessLedCount = 0;
reader.blinkWirelessLed = function()
{
    // Blink the wireless LED
    if (reader.blinkWirelessLedCount > 0)
        reader.blinkWirelessLedCount--;
    reader.indicateWirelessLed(reader.blinkWirelessLedCount % 2);
    if (reader.blinkWirelessLedCount <= 0)
    {
        if (reader.blinkWirelessLedIntervalId >= 0)
        {
            clearInterval(reader.blinkWirelessLedIntervalId);
            reader.blinkWirelessLedIntervalId = -1;
        }
    }
}

reader.blinkBatteryLedIntervalId = -1;
reader.blinkBatteryLed = function()
{
    reader.indicateBatteryLed(1);
}

reader.setSleepBlink = function(setting, value)
{
    if( reader.readSetting(setting) != value )
    {
        reader.writeSetting(setting, value);

        if( value != 0 )
        {
            // Now sync up all of the counters
            reader.writeSetting(setting_sleepGoodLedSyncPosition,     -1);
            reader.writeSetting(setting_sleepErrorLedSyncPosition,    -1);
            reader.writeSetting(setting_sleepDataLedSyncPosition,     -1);
            reader.writeSetting(setting_sleepWirelessLedSyncPosition, -1);
            reader.writeSetting(setting_sleepBatteryLedSyncPosition,  -1);
        }
    }
}


reader.pagingModeIntervalId  = -1;
reader.pagingModeStartTime   = 0;
reader.pagingModeTimeout_sec = 30;
reader.pagingModePeriod_ms   = 1000;

reader.cancelPagingMode = function()
{
    if( reader.pagingModeIntervalId != -1 )
    {
        reader.writeSetting(setting_button1_event, reader.pagingModeButton1);
        reader.writeSetting(setting_button2_event, reader.pagingModeButton2);
        reader.writeSetting(setting_button4_event, reader.pagingModeButton4);
        reader.clearInterval(reader.pagingModeIntervalId);
        reader.pagingModeIntervalId = -1;
    }
}

reader.periodicPage = function()
{
    var now = new Date();
    var sec = now.getTime() / 1000;
    if( sec - reader.pagingModeStartTime > reader.pagingModeTimeout_sec )
    {
        reader.cancelPagingMode();
    }
    else
    {
        reader.beep(1);
        reader.processCommand('$',"\x03");
    }
}



reader.onPaging = function()
{
    if( reader.pagingModeIntervalId != -1 )
    {
        reader.cancelPagingMode();
    }
    else
    {
        reader.pagingModeButton1 = reader.readSetting(setting_button1_event);
        reader.pagingModeButton2 = reader.readSetting(setting_button2_event);
        reader.pagingModeButton4 = reader.readSetting(setting_button4_event);
        reader.writeSetting(setting_button1_event, event_paging);
        reader.writeSetting(setting_button2_event, event_paging);
        reader.writeSetting(setting_button4_event, event_paging);

        var now = new Date();
        var sec = now.getTime() / 1000;

        reader.pagingModeStartTime = sec;
        reader.pagingModeIntervalId = setInterval(reader.periodicPage, reader.pagingModePeriod_ms);
        reader.periodicPage();
    }
}


reader.onEvent = function(type)
{
    var result = true;

    if (rules_onEvent != null)
        result = rules_onEvent(type);

    if (result == true)
    {
        //Specific event handling
        if     ( event_cradled   == type )
        {
            if( reader.blinkBatteryLedIntervalId == -1 )
            {
                reader.indicateBatteryLed(1);
                reader.blinkBatteryLedIntervalId = reader.setInterval(reader.blinkBatteryLed, 5);
            }
        }
        else if( event_uncradled == type )
        {
            if( reader.blinkBatteryLedIntervalId != -1 )
            {
                reader.clearInterval(reader.blinkBatteryLedIntervalId);
                reader.blinkBatteryLedIntervalId = -1;
            }
        }
        else if( event_paging    == type )
        {
            reader.onPaging();
        }
        else if( event_appleMode == type )
        {
            if (commMode_btKb == reader.readSetting(setting_commMode))
            {
                // Apple keyboard HID ReportID = 0x11 Data=0x08 (press) 0x00 (release)
                //  to toggle on screen keyboard
                comm.sendHidReport(0x11, "\x08");
                comm.sendHidReport(0x11, "\x00");
            }
            else
            {
                reader.processCommand("$", String.fromCharCode(reader.readSetting(setting_appleMode_event)));
            }
        }
    }

    if(!result)
        return false;
}



reader.onPrefixSuffixChange = function()
{
    store.updatePrefixSuffix();
}


reader.setTimeout = function(func, delay_sec)
{
    return setTimeout(func,delay_sec * 1000);
}

reader.setInterval = function(func, interval_sec)
{
    return setInterval(func, interval_sec * 1000);
}

reader.clearTimeout = function(timeoutId)
{
    clearTimeout(timeoutId);
}

reader.clearInterval = function(intervalId)
{
    clearInterval(intervalId);
}

///////////////////////////////////////////////////////////////
// main code (startup)

store.init();
ready.show();


///////////////////////////////////////////////////////////////
// EOF
