/* .jseLib.js
 *
 * $Revision: 7022 $
 * Copyright 2006 The Code Corporation. All rights are reserved.
 */



////////////////////////////////////////////////////////////////////////
// CR3 core objects



// constructors
gui.Form            = _Gui_List;
gui.Menu            = _Gui_List;
gui.MenuItem        = _Gui_MenuItem;
gui.Button          = _Gui_MenuItem;
gui.Edit            = _Gui_Edit;
gui.MultiLineEdit   = _Gui_MultiLineEdit;
gui.Label           = _Gui_StaticText;
gui.Softkey         = _Gui_Softkey;
gui.Text            = _Gui_Text;
gui.Separator       = _Gui_Separator;
gui.Image           = _Gui_Image;


gui._currentForm = null;
gui.leftSoftkey  = null;
gui.rightSoftkey = null;



// show
function _gui_show(f)
{
    gui.currentForm = f;

    gui.showNative(f);
}

gui.show = _gui_show;




// storage
function _storage_findFirst(re)
{
    storage._findRe = re;

    var name = storage.findFirstNative();

    while( name != null )
    {
        if( storage._findRe.test(name) )
            return name;
        else
            name = storage.findNextNative();
    }
    return null;
}



// storage
function _storage_findFirstIn(re, dirNum)
{
    storage._findRe = re;

    var name = storage.findFirstInNative(dirNum);

    while( name != null )
    {
        if( storage._findRe.test(name) )
            return name;
        else
            name = storage.findNextNative();
    }
    return null;
}



function _storage_findNext()
{
    var name = storage.findNextNative();

    while( name != null )
    {
        if( storage._findRe.test(name) )
            return name;
        else
            name = storage.findNextNative();
    }
}



storage.findFirst = _storage_findFirst;
storage.findNext  = _storage_findNext;
storage.findFirstIn = _storage_findFirstIn;



// append/prepend
function _List_append(i)
{
    this.children[this.children.length] = i;
    return this.appendNative(i);
}



function _List_prepend(i)
{
    for( i = this.children.length; i > 0; i-- )
    {
        this.children[i] = this.children[i - 1];
    }

    this.children[0] = i;

    return this.prependNative(i);
}



_Gui_List.prototype.children = new Array();

_Gui_List.prototype.append = _List_append;
_Gui_List.prototype.prepend = _List_prepend;



////////////////////////////////////////////////////////////////////////
// additional gui support, including form, menu, softkeys



// define send-key functions used by common softkeys

gui.sendEnter  = function() { gui.sendKey(gui.key.enter ); }
gui.sendEscape = function() { gui.sendKey(gui.key.escape); }



// create some common softkeys

gui.selectSoftkey = new gui.Softkey("Select", gui.sendEnter );
gui.okSoftkey     = new gui.Softkey("OK",     gui.sendEnter );
gui.backSoftkey   = new gui.Softkey("Back",   gui.sendEscape);
gui.cancelSoftkey = new gui.Softkey("Cancel", gui.sendEscape);



gui.showMenu = function(menu)
{
    // shows form along with appropriate softkeys
    // also shows caption, which is optional

    gui.statusText   = menu.caption;
    gui.leftSoftkey  = gui.selectSoftkey;
    gui.rightSoftkey = menu.onCancel? gui.backSoftkey: null;

    gui.show(menu);
}



gui.showSubMenu = function(menu, parentMenu)
{
    // links menu to parentMenu and shows it along with appropriate softkeys (and caption)

    menu.onCancel = function() { gui.showMenu(parentMenu); return true; };
    gui.showMenu(menu);
}



gui.showForm = function(form)
{
    // shows form along with appropriate softkeys;
    // also shows caption, which is optional

    gui.leftSoftkey  = form.leftSoftkey?  form.leftSoftkey:  gui.okSoftkey;
    gui.rightSoftkey = form.rightSoftkey? form.rightSoftkey: gui.cancelSoftkey;
    gui.statusText   = form.caption;

    gui.show(form);
}



gui.showDialog = function(dialog, caption)
{
    // shows dialog, including softkeys and caption

    gui.leftSoftkey  = dialog.leftSoftkey;
    gui.rightSoftkey = dialog.rightSoftkey;
    gui.statusText   = dialog.caption;

    gui.show(dialog);
}



// ToggleButton class
function _ToggleButton_toggle()
{
    // note: character 0x1f has been defined as a space of same width as '*'

    var checked = this.text.charAt(0) == '*';
    var text    = this.text.substring(1, this.text.length);
    checked     = !checked;
    this.text   = (checked? "*": "\x1f") + text;
    this.checked = checked;
    return this.onToggle(checked, this.text.substring(2/* excluding the space or '*' */));
}



function _ToggleButton_check(checked)
{
    var text  = this.text.substring(1, this.text.length);
    this.text = (checked? "*": "\x1f") + text;
    this.checked = checked;
    return this.onToggle(checked, this.text.substring(2/* excluding the space or '*' */));
}



gui.ToggleButton = function(text, checked, onToggle)
{
    button = new gui.Button((checked? "*": "\x1f") + text, _ToggleButton_toggle);

    button.check = _ToggleButton_check;
    button.toggle = _ToggleButton_toggle;

    button.checked = checked;

    button.onToggle = onToggle;

    return button;
}


/*****************************************************************************/
// Alert/Prompt/Confirm Forms
/****************************************************************************/

//Declare some global variables
var Yes_Confirmation_Func = null; //function that will be called once user provides yes input.
var No_Confirmation_Func = null;  //function that will be called once user provides no input.
var Alert_Confirmation_Func = null; //function that will be called once user provides an input.
var Prompt_Confirmation_Func = null //function that will be called once user provides an input.

function alert(func, s)
{
    gui.alert(func, s);
}



gui.alert = function(func, s, title)
{
	Alert_Confirmation_Func = func;
	gui.alertEx(s, title);
}



gui.alertEx = function(s, title)
{
    if( !title && title != "" )
        title = "Alert";

    function OnOk_Click()
    {
		gui.leftSoftkey  = alert.prevLeftSoftkey;
		gui.rightSoftkey = alert.prevRightSoftkey;
		gui.statusText   = alert.prevStatusText;
		gui.show(alert.prevForm);
		if(Alert_Confirmation_Func != null)
		{
			Alert_Confirmation_Func();
		}
    }

    alert.prevForm         = gui.currentForm;
    alert.prevLeftSoftkey  = gui.leftSoftkey;
    alert.prevRightSoftkey = gui.rightSoftkey;
    alert.prevStatusText   = gui.statusText;

    alert.f = new gui.Text(s.toString());

    gui.leftSoftkey  = new gui.Softkey("OK", OnOk_Click);
	gui.rightSoftkey = null;
    gui.statusText   = title;

    gui.show(alert.f);
}



function confirm(yesFunc, noFunc, s)
{
    return gui.confirm(yesFunc, noFunc, s, null, "OK", "Cancel");
}



//Since jsEngine runs in the main thread context
//if we block in jsEngine, it stalls everything and
//system becomes unresponsive.  So, here we do an indirect
//event generation and call appropriate function based on
//user input.
gui.confirm = function(yesFunc, noFunc, s, title, yesText, noText)
{
    //Call this function once user has provided an input.
    Yes_Confirmation_Func = yesFunc;
    No_Confirmation_Func = noFunc;
    gui.confirmEx(s, title, yesText, noText);
}



gui.confirmEx = function(s, title, yesText, noText)
{
    if( !title && title != "" )
        title = "Confirm";
    if( !yesText )
        yesText = "Yes";
    if( !noText )
        noText = "No";

    function OnYes_Click()
    {
        //Restore the previous form
        gui.leftSoftkey  = confirm.prevLeftSoftkey;
        gui.rightSoftkey = confirm.prevRightSoftkey;
        gui.statusText   = confirm.prevStatusText;
        gui.show(confirm.prevForm);
        //Call the yes function that was passed to the confirm form.
		if(Yes_Confirmation_Func != null)
		{
			Yes_Confirmation_Func();
		}
    }

    function OnNo_Click()
    {
        //Restore the previous form
        gui.leftSoftkey  = confirm.prevLeftSoftkey;
        gui.rightSoftkey = confirm.prevRightSoftkey;
        gui.statusText   = confirm.prevStatusText;
        gui.show(confirm.prevForm);
        //Call the no function that was passed to the confirm form.
		if(No_Confirmation_Func != null)
		{
			No_Confirmation_Func();
		}
    }

    confirm.prevForm         = gui.currentForm;
    confirm.prevLeftSoftkey  = gui.leftSoftkey;
    confirm.prevRightSoftkey = gui.rightSoftkey;
    confirm.prevStatusText   = gui.statusText;

    confirm.f = new gui.Text(s.toString(), OnYes_Click, OnNo_Click);

    gui.leftSoftkey  = new gui.Softkey(yesText, OnYes_Click);
    gui.rightSoftkey = new gui.Softkey(noText,  OnNo_Click);
    gui.statusText   = title;

    gui.show(confirm.f);
}



function prompt(func, s, def)
{
    return gui.prompt(func, s, def);
}



gui.prompt = function (func, s, def, title)
{
	Prompt_Confirmation_Func = func;
	gui.promptEx(s, def, title);
}



gui.promptEx = function(s, def, title)
{
    if( !title && title != "" )
        title = "Prompt";

    function onOk()
    {
		//Restore the previous form
		gui.leftSoftkey  = prompt.prevLeftSoftkey;
		gui.rightSoftkey = prompt.prevRightSoftkey;
		gui.statusText   = prompt.prevStatusText;
		gui.show(prompt.prevForm);
		//Call the function which was passed to prompt form
		//and return the text entered in the edit control
		if(Prompt_Confirmation_Func != null)
		{
			Prompt_Confirmation_Func(prompt.e.text);
		}
    }

    function onCancel()
    {
		//Restore the previous form
        gui.leftSoftkey  = prompt.prevLeftSoftkey;
		gui.rightSoftkey = prompt.prevRightSoftkey;
		gui.statusText   = prompt.prevStatusText;
		gui.show(prompt.prevForm);
		//Call the function which was passed to prompt form
		//and return null
		if(Prompt_Confirmation_Func != null)
		{
			Prompt_Confirmation_Func(null);
		}
    }

    prompt.prevForm         = gui.currentForm;
    prompt.prevLeftSoftkey  = gui.leftSoftkey;
    prompt.prevRightSoftkey = gui.rightSoftkey;
    prompt.prevStatusText   = gui.statusText;

    prompt.f = new gui.Form(onOk, onCancel);

    prompt.f.append(new gui.Label(s.toString()));

    prompt.e = new gui.Edit(def);
    prompt.f.append(prompt.e);

    gui.leftSoftkey  = gui.okSoftkey;
    gui.rightSoftkey = gui.cancelSoftkey;
    gui.statusText   = title;

    gui.show(prompt.f);
}



////////////////////////////////////////////////////////////////////
// setTimeout and setInterval
var _timerInternalId_ = 0;
_timers = new Array();

function Timer(func, interval_ms, repeat, timerId)
{
    this.timerId     = timerId;
    this.func        = func;
    this.interval_ms = interval_ms;
    this.repeat      = repeat;
}



function setTimeout(func, delay_ms)
{
    var timer = new Timer(func, delay_ms, false, _timerInternalId_);
    _timers.push(timer);
    _setTimer(timer.timerId, delay_ms);
    _timerInternalId_++;
    return timer.timerId;
}



function setInterval(func, delay_ms)
{
    var timer = new Timer(func, delay_ms, true, _timerInternalId_);
    _timers.push(timer);
    _setTimer(timer.timerId, delay_ms);
    _timerInternalId_++;
    return timer.timerId;
}



function _removeTimer(timerId)
{
    for (var i = 0; i < _timers.length; i++)
    {
        if (_timers[i].timerId == timerId)
        {
           _timers.splice(i, 1);
           break;
        }
    }
}



function clearTimeout(id)
{
    _stopTimer(id);
    _removeTimer(id);
}



function clearInterval(id)
{
    clearTimeout(id);
}



function _timerFired(timerId)
{
    for( var i = 0; i < _timers.length; ++i )
    {
        if( _timers[i].timerId == timerId )
        {
            removeThisTimer = 0;
            if( _timers[i].repeat )
                _setTimer(_timers[i].timerId, _timers[i].interval_ms);
            else
                removeThisTimer = 1;

            _timers[i].func();

            // Don't access _timers[i] after calling the function, in case the
            //  timer function already removed its own timer
            if( 1 == removeThisTimer )
            {
                _removeTimer(timerId);
            }

            break;
        }
    }
}



//EOF
