/* Filename:    J007_SimpleInv7.js
 * Description: Creating a simple inventory application -
 *              Add buttons to form, also set input modes on edit fields
 *
 *              Input to the Simlated Decode on the jsSim3600.js is:
 *
 *              MMMPPP
 *
 *              where MMM = Manufacturer Number and PPP = Part Number.
 *              Quantity is only accessable through the GUI
 *
 * Copyright:   2014 The Code Corporation.  All rights are reserved.
 * $Revision: 636 $
 * $Date: 2014-02-26 17:24:04 -0700 (Wed, 26 Feb 2014) $
 */
 
// Constants
var sepHeight = 8;

// Helper functions
function focusQuantity()
{
    // Set up the reader to input the quantity
    inventory.setActiveChild(quantityEdit); // After settingSetActiveChild you must re-show the form
    gui.showForm(inventory);
}

// Form event handlers
function cancel()
{
    // Return to default application
    reader.runScript(".default.js");
}

function ok()
{
    // Write to inventory.dat file
    if( quantityEdit.text == "" && (manufacturerEdit.text != "" ||  partEdit.text != ""))
    {
        focusQuantity();
        return;
    }
    else
    {
        var s = "MF:" + manufacturerEdit.text + "\t" + "PN:" + partEdit.text + "\t" + "QT:" + quantityEdit.text + "\n";
        
        gui.statusText = "Saving...";
        
        if( storage.append("inventory.dat", s) )
        {
            gui.statusText = "Saved";
            
            manufacturerEdit.text = "";
            partEdit.text         = "";
            quantityEdit.text     = "";
            
            inventory.setActiveChild(manufacturerLabel);
        }
        else
        {
            gui.statusText = "Save failed!";
        }
    }
    gui.showForm(inventory);
}

// Reader interface functions
reader.onDecode = function(decode)
{
    // Split decoded data into 2 edit fields
    manufacturerEdit.text = decode.data.substr(0, 3);
    partEdit.text         = decode.data.substr(3, 3);
    
    focusQuantity();
}

// Variables
var inventory         = new gui.Form(ok , cancel);
var manufacturerLabel = new gui.Label("Manuf. ID:"); // Name this label for use in setActiveChild
var manufacturerEdit  = new gui.Edit("", gui.inputMode.caps);
var partEdit          = new gui.Edit("", gui.inputMode.numeric);
var quantityEdit      = new gui.Edit("", gui.inputMode.numeric, gui.inputMode.numeric);

// Add a caption
inventory.caption    = "Inventory";

// Add label and edit boxes to the form
inventory.append(manufacturerLabel);
inventory.append(manufacturerEdit);
inventory.append(new gui.Label("Part #:")); // Append a new label directly when access to the element is not needed
inventory.append(partEdit);
inventory.append(new gui.Label("Quantity:"));
inventory.append(quantityEdit);

// Add buttons (and separator)
inventory.append(new gui.Separator(sepHeight, gui.separatorStyle.horizontalLine));
inventory.append(new gui.Button("Save", ok));
inventory.append(new gui.Button("Upload all", function() { storage.upload("inventory.dat"); })); // In-line a simple function

// Show the form
gui.showForm(inventory);

//EOF